@@ -1,6 +1,6 @@
 package Dancer2;
 # ABSTRACT: Lightweight yet powerful web application framework
-$Dancer2::VERSION = '0.158000';
+$Dancer2::VERSION = '0.159000';
 use strict;
 use warnings;
 use List::Util  'first';
@@ -16,7 +16,7 @@ our $AUTHORITY = 'SUKRIA';
 # set version in dist.ini now
 # but we still need a basic version for
 # the tests
-$Dancer2::VERSION ||= '0.158000';
+$Dancer2::VERSION ||= '0.159000';
 
 our $runner;
 
@@ -117,7 +117,7 @@ Dancer2 - Lightweight yet powerful web application framework
 
 =head1 VERSION
 
-version 0.158000
+version 0.159000
 
 =head1 DESCRIPTION
 
@@ -162,14 +162,19 @@ information on the concepts of Dancer2 application development and
 a comprehensive reference to the Dancer2 domain specific
 language.
 
+=item * Dancer2 Keywords
+
+The keywords for Dancer2 can be found under L<DSL Keywords|Dancer2::Manual/DSL KEYWORDS>.
+
+=item * Dancer2 Deployment
+
+For configuration examples of different deployment solutions involving
+Dancer2 and Plack, refer to L<Dancer2::Manual::Deployment>.
+
 =item * Dancer2 Cookbook
 
-There are some situations that are common to a lot of users. For
-example, application deployment. in L<Dancer2::Cookbook> you will find
-recipes for common tasks, like defining routes, storing data as
-sessions or cookies, using templates, configuring and logging, writing
-REST services, and deploying your Dancer2 application using different
-technologies.
+Specific examples of code for real-life problems and some 'tricks' for
+applications in Dancer can be found in <Dancer2::Cookbook>
 
 =item * Dancer2 Config
 
