@@ -1,6 +1,6 @@
 # ABSTRACT: encapsulation of Dancer2 packages
 package Dancer2::Core::App;
-$Dancer2::Core::App::VERSION = '0.158000';
+$Dancer2::Core::App::VERSION = '0.159000';
 use Moo;
 use Carp               'croak';
 use Scalar::Util       'blessed';
@@ -511,6 +511,9 @@ around execute_hook => sub {
     my $orig = shift;
     my $self = shift;
 
+    local $Dancer2::Core::Route::REQUEST  = $self->request;
+    local $Dancer2::Core::Route::RESPONSE = $self->response;
+
     my ( $hook, @args ) = @_;
     if ( !$self->has_hook($hook) ) {
         foreach my $cand ( $self->hook_candidates ) {
@@ -557,7 +560,7 @@ sub _init_hooks {
         Dancer2::Core::Hook->new(
             name => 'core.app.after_request',
             code => sub {
-                my $response = $app->response;
+                my $response = $Dancer2::Core::Route::RESPONSE;
 
                 # make sure an engine is defined, if not, nothing to do
                 my $engine = $app->session_engine;
@@ -668,6 +671,12 @@ sub template {
     my $template = $self->template_engine;
     $template->set_settings( $self->config );
 
+    # A session may exist but the route code may not have instantiated
+    # the session object (sessions are lazy). If this is the case, do
+    # that now, so the templates have the session data for rendering.
+    $self->has_session && ! $template->has_session
+        and $self->setup_session;
+
     # return content
     return $template->process( @_ );
 }
@@ -1077,6 +1086,7 @@ sub to_app {
             $psgi,
             path => sub { -f path( $self->config->{public_dir}, shift ) },
             root => $self->config->{public_dir},
+            content_type => sub { $self->mime_type->for_name(shift) },
         ) },
         condition => sub { $self->config->{static_handler} },
     );
@@ -1197,9 +1207,8 @@ DISPATCH:
         last;
     }
 
-    $self->cleanup;
 
-    # make sure Core::Dispatcher recognizes this failure
+    # No response! ensure Core::Dispatcher recognizes this failure
     # so it can try the next Core::App
     # and set the created request so we don't create it again
     # (this is important so we don't ignore the previous body)
@@ -1208,7 +1217,10 @@ DISPATCH:
         Dancer2->runner->{'internal_request'} = $request;
     }
 
-    return $self->response_not_found($request);
+    # Render 404 response, cleanup, and return the response.
+    my $response = $self->response_not_found($request);
+    $self->cleanup;
+    return $response;
 }
 
 sub build_request {
@@ -1232,41 +1244,39 @@ sub _dispatch_route {
     $self->execute_hook( 'core.app.before_request', $self );
     my $response = $self->response;
 
-    my $content;
     if ( $response->is_halted ) {
-        # if halted, it comes from the 'before' hook. Take its content
-        $content = $response->content;
+        return $self->_add_content_to_response(
+            $response, $response->content,
+        );
     }
-    else {
-        $content = eval { $route->execute($self) };
 
+    $response = eval {
+        $route->execute($self)
+    } or do {
         my $error = $@;
-        if ($error) {
-            $self->log( error => "Route exception: $error" );
-            $self->execute_hook( 'core.app.route_exception', $self, $error );
-            return $self->response_internal_error($error);
-        }
-    }
+        $self->log( error => "Route exception: $error" );
+        $self->execute_hook( 'core.app.route_exception', $self, $error );
+        return $self->response_internal_error($error);
+    };
 
-    $response->has_content
-        and $content = $response->content;
+    return $response;
+}
 
-    if ( ref $content eq 'Dancer2::Core::Response' ) {
-        $response = $self->set_response($content);
-    }
-    elsif ( defined $content ) {
-        # The response object has no back references to the content or app
-        # Update the default_content_type of the response if any value set in
-        # config so it can be applied when the response is encoded/returned.
-        if ( exists $self->config->{content_type}
-          && $self->config->{content_type} ) {
-            $response->default_content_type($self->config->{content_type});
-        }
+sub _add_content_to_response {
+    my ( $self, $response, $content ) = @_;
 
-        $response->content($content);
-        $response->encode_content;
+    defined $content or return $response;
+
+    # The response object has no back references to the content or app
+    # Update the default_content_type of the response if any value set in
+    # config so it can be applied when the response is encoded/returned.
+    if ( exists $self->config->{content_type}
+      && $self->config->{content_type} ) {
+        $response->default_content_type($self->config->{content_type});
     }
 
+    $response->content($content);
+    $response->encode_content;
     return $response;
 }
 
@@ -1312,7 +1322,7 @@ Dancer2::Core::App - encapsulation of Dancer2 packages
 
 =head1 VERSION
 
-version 0.158000
+version 0.159000
 
 =head1 DESCRIPTION
 
