@@ -1,12 +1,13 @@
 # ABSTRACT: Dancer2's Domain Specific Language (DSL)
 
 package Dancer2::Core::DSL;
-$Dancer2::Core::DSL::VERSION = '0.157001';
+$Dancer2::Core::DSL::VERSION = '0.159000';
 use Moo;
 use Carp;
 use Class::Load 'load_class';
 use Dancer2::Core::Hook;
 use Dancer2::FileUtils;
+use Dancer2::Core::Response::Delayed;
 
 with 'Dancer2::Core::Role::DSL';
 
@@ -30,11 +31,16 @@ sub dsl_keywords {
         dancer_major_version => { is_global => 1 },
         debug                => { is_global => 1 },
         del                  => { is_global => 1 },
+        delayed              => {
+            is_global => 0, prototype => '&',
+        },
         dirname              => { is_global => 1 },
+        done                 => { is_global => 0 },
         dsl                  => { is_global => 1 },
         engine               => { is_global => 1 },
         error                => { is_global => 1 },
         false                => { is_global => 1 },
+        flush                => { is_global => 0 },
         forward              => { is_global => 0 },
         from_dumper          => { is_global => 1 },
         from_json            => { is_global => 1 },
@@ -165,102 +171,40 @@ sub prefix {
 
 sub halt { shift->app->halt }
 
-sub _route_parameters {
-    my ( $regexp, $code, $options );
-    ( scalar @_ == 3 )
-      ? ( ( $regexp, $code, $options ) = ( $_[0], $_[2], $_[1] ) )
-      : ( ( $regexp, $code, $options ) = ( $_[0], $_[1], {} ) );
-    return ( $regexp, $code, $options );
-}
-
-sub get {
-    my $app = shift->app;
-
-    my ( $regexp, $code, $options ) = _route_parameters(@_);
-    for my $method (qw/get head/) {
-        $app->add_route(
-            method  => $method,
-            regexp  => $regexp,
-            code    => $code,
-            options => $options
-        );
-    }
-}
-
-sub post {
-    my $app = shift->app;
-
-    my ( $regexp, $code, $options ) = _route_parameters(@_);
-    $app->add_route(
-        method  => 'post',
-        regexp  => $regexp,
-        code    => $code,
-        options => $options
-    );
-}
+sub del     { shift->_normalize_route( [qw/delete  /], @_ ) }
+sub get     { shift->_normalize_route( [qw/get head/], @_ ) }
+sub options { shift->_normalize_route( [qw/options /], @_ ) }
+sub patch   { shift->_normalize_route( [qw/patch   /], @_ ) }
+sub post    { shift->_normalize_route( [qw/post    /], @_ ) }
+sub put     { shift->_normalize_route( [qw/put     /], @_ ) }
 
 sub any {
-    my ( $self, $methods, @params ) = @_;
-    my $app = $self->app;
+    my $self = shift;
 
-    if ($methods) {
-        if ( ref($methods) ne 'ARRAY' ) {
-            unshift @params, $methods;
-            $methods = [qw(get post put del options patch)];
-        }
+    # If they've supplied their own list of methods,
+    # expand del, otherwise give them the default list.
+    if ( ref $_[0] eq 'ARRAY' ) {
+        s/^del$/delete/ for @{ $_[0] };
+    }
+    else {
+        unshift @_, [qw/delete get head options patch post put/];
     }
 
-    for my $method ( @{$methods} ) {
-        $self->$method(@params);
-    }
+    $self->_normalize_route(@_);
 }
 
-sub put {
-    my $app = shift->app;
+sub _normalize_route {
+    my $app     = shift->app;
+    my $methods = shift;
+    my %args;
 
-    my ( $regexp, $code, $options ) = _route_parameters(@_);
-    $app->add_route(
-        method  => 'put',
-        regexp  => $regexp,
-        code    => $code,
-        options => $options,
-    );
-}
+    # Options are optional, deduce their presence from arg length.
+    # @_ = ( REGEXP, OPTIONS, CODE )
+    # or
+    # @_ = ( REGEXP, CODE )
+    @args{qw/regexp options code/} = @_ == 3 ? @_ : ( $_[0], {}, $_[1] );
 
-sub del {
-    my $app = shift->app;
-
-    my ( $regexp, $code, $options ) = _route_parameters(@_);
-    $app->add_route(
-        method  => 'delete',
-        regexp  => $regexp,
-        code    => $code,
-        options => $options,
-    );
-}
-
-sub options {
-    my $app = shift->app;
-
-    my ( $regexp, $code, $options ) = _route_parameters(@_);
-    $app->add_route(
-        method  => 'options',
-        regexp  => $regexp,
-        code    => $code,
-        options => $options,
-    );
-}
-
-sub patch {
-    my $app = shift->app;
-
-    my ( $regexp, $code, $options ) = _route_parameters(@_);
-    $app->add_route(
-        method  => 'patch',
-        regexp  => $regexp,
-        code    => $code,
-        options => $options,
-    );
+    $app->add_route( %args, method => $_ ) for @{$methods};
 }
 
 #
@@ -289,12 +233,92 @@ sub to_app { shift->app->to_app }
 # Response alterations
 #
 
-sub status       { shift->response->status(@_) }
-sub push_header  { shift->response->push_header(@_) }
-sub header       { shift->response->header(@_) }
-sub headers      { shift->response->header(@_) }
-sub content      { shift->response->content(@_) }
-sub content_type { shift->response->content_type(@_) }
+sub status {
+    $Dancer2::Core::Route::RESPONSE->status( $_[1] );
+}
+
+sub push_header {
+    shift;
+    $Dancer2::Core::Route::RESPONSE->push_header(@_);
+}
+
+sub header {
+    shift;
+    $Dancer2::Core::Route::RESPONSE->header(@_);
+}
+
+sub headers {
+    shift;
+    $Dancer2::Core::Route::RESPONSE->header(@_);
+}
+
+sub content {
+    shift;
+
+    # simple synchronous response
+    my $responder = $Dancer2::Core::Route::RESPONDER
+        or croak 'Cannot use content keyword outside delayed response';
+
+    # flush if wasn't flushed before
+    if ( !$Dancer2::Core::Route::WRITER ) {
+        my $response = $Dancer2::Core::Route::RESPONSE;
+        $Dancer2::Core::Route::WRITER = $responder->([
+            $response->status, $response->headers_to_array,
+        ]);
+    }
+
+    $Dancer2::Core::Route::WRITER->write(@_);
+}
+
+sub content_type {
+    shift;
+    $Dancer2::Core::Route::RESPONSE->content_type(@_);
+}
+
+sub delayed {
+    my ( $dsl, $cb ) = @_;
+
+    # first time, responder doesn't exist yet
+    $Dancer2::Core::Route::RESPONDER
+        or return Dancer2::Core::Response::Delayed->new(
+            cb       => $cb,
+            request  => $Dancer2::Core::Route::REQUEST,
+            response => $Dancer2::Core::Route::RESPONSE,
+        );
+
+    # we're in an async request process
+    my $request   = $Dancer2::Core::Route::REQUEST;
+    my $response  = $Dancer2::Core::Route::RESPONSE;
+    my $responder = $Dancer2::Core::Route::RESPONDER;
+    my $writer    = $Dancer2::Core::Route::WRITER;
+
+    return sub {
+        local $Dancer2::Core::Route::REQUEST   = $request;
+        local $Dancer2::Core::Route::RESPONSE  = $response;
+        local $Dancer2::Core::Route::RESPONDER = $responder;
+        local $Dancer2::Core::Route::WRITER    = $writer;
+
+        $cb->(@_);
+    };
+}
+
+sub flush {
+    my $responder = $Dancer2::Core::Route::RESPONDER
+        or croak 'flush() called outside streaming response';
+
+    my $response = $Dancer2::Core::Route::RESPONSE;
+    $Dancer2::Core::Route::WRITER = $responder->([
+        $response->status, $response->headers_to_array,
+    ]);
+}
+
+sub done {
+    my $writer = $Dancer2::Core::Route::WRITER
+        or croak 'done() called outside streaming response';
+
+    $writer->close;
+}
+
 sub pass         { shift->app->pass }
 
 #
@@ -306,30 +330,31 @@ sub context {
     shift->app;
 }
 
-sub request { shift->app->request }
+sub request { $Dancer2::Core::Route::REQUEST }
 
-sub response { shift->app->response }
+sub response { $Dancer2::Core::Route::RESPONSE }
 
-sub upload { shift->request->upload(@_) }
+sub upload { shift; $Dancer2::Core::Route::REQUEST->upload(@_); }
 
-sub captures { shift->request->captures }
+sub captures { $Dancer2::Core::Route::REQUEST->captures }
 
-sub uri_for { shift->request->uri_for(@_) }
+sub uri_for { shift; $Dancer2::Core::Route::REQUEST->uri_for(@_); }
 
-sub splat { shift->request->splat }
+sub splat { $Dancer2::Core::Route::REQUEST->splat }
 
-sub params { shift->request->params(@_) }
+sub params { shift; $Dancer2::Core::Route::REQUEST->params(@_); }
 
-sub param { shift->request->param(@_) }
+sub param { shift; $Dancer2::Core::Route::REQUEST->param(@_); }
 
 sub redirect { shift->app->redirect(@_) }
 
 sub forward { shift->app->forward(@_) }
 
-sub vars { shift->request->vars }
-sub var  { shift->request->var(@_) }
+sub vars { $Dancer2::Core::Route::REQUEST->vars }
 
-sub cookies { shift->request->cookies }
+sub var  { shift; $Dancer2::Core::Route::REQUEST->var(@_); }
+
+sub cookies { $Dancer2::Core::Route::REQUEST->cookies }
 sub cookie { shift->app->cookie(@_) }
 
 sub mime {
@@ -350,37 +375,37 @@ sub mime {
 
 sub from_json {
     shift; # remove first element
-    require 'Dancer2/Serializer/JSON.pm';
+    require Dancer2::Serializer::JSON;
     Dancer2::Serializer::JSON::from_json(@_);
 }
 
 sub to_json {
     shift; # remove first element
-    require 'Dancer2/Serializer/JSON.pm';
+    require Dancer2::Serializer::JSON;
     Dancer2::Serializer::JSON::to_json(@_);
 }
 
 sub from_yaml {
     shift; # remove first element
-    require 'Dancer2/Serializer/YAML.pm';
+    require Dancer2::Serializer::YAML;
     Dancer2::Serializer::YAML::from_yaml(@_);
 }
 
 sub to_yaml {
     shift; # remove first element
-    require 'Dancer2/Serializer/YAML.pm';
+    require Dancer2::Serializer::YAML;
     Dancer2::Serializer::YAML::to_yaml(@_);
 }
 
 sub from_dumper {
     shift; # remove first element
-    require 'Dancer2/Serializer/Dumper.pm';
+    require Dancer2::Serializer::Dumper;
     Dancer2::Serializer::Dumper::from_dumper(@_);
 }
 
 sub to_dumper {
     shift; # remove first element
-    require 'Dancer2/Serializer/Dumper.pm';
+    require Dancer2::Serializer::Dumper;
     Dancer2::Serializer::Dumper::to_dumper(@_);
 }
 
@@ -400,7 +425,7 @@ Dancer2::Core::DSL - Dancer2's Domain Specific Language (DSL)
 
 =head1 VERSION
 
-version 0.157001
+version 0.159000
 
 =head1 FUNCTIONS
 
@@ -429,7 +454,7 @@ Dancer Core Developers
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Alexis Sukrieh.
+This software is copyright (c) 2015 by Alexis Sukrieh.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
