@@ -1,6 +1,6 @@
 package Dancer2::Core::Error;
 # ABSTRACT: Class representing fatal errors
-$Dancer2::Core::Error::VERSION = '0.157001';
+$Dancer2::Core::Error::VERSION = '0.159000';
 use Moo;
 use Carp;
 use Dancer2::Core::Types;
@@ -85,8 +85,7 @@ sub _build_static_page {
 
     # TODO there must be a better way to get it
     my $public_dir = $ENV{DANCER_PUBLIC}
-      || ( $self->has_app
-        && path( $self->app->config_location, 'public' ) );
+      || ( $self->has_app && $self->app->config->{public_dir} );
 
     my $filename = sprintf "%s/%d.html", $public_dir, $self->status;
 
@@ -206,7 +205,13 @@ has exception => (
 has response => (
     is      => 'rw',
     lazy    => 1,
-    default => sub { Dancer2::Core::Response->new }
+    default => sub {
+        my $self = shift;
+        my $serializer = $self->serializer;
+        return Dancer2::Core::Response->new(
+            ( serializer => $serializer )x!! $serializer
+        );
+    }
 );
 
 has content_type => (
@@ -223,52 +228,71 @@ has content_type => (
 has content => (
     is      => 'ro',
     lazy    => 1,
-    default => sub {
-        my $self = shift;
-
-        # Apply serializer
-        if ( $self->serializer ) {
-            my $content = {
-                message => $self->message,
-                title   => $self->title,
-                status  => $self->status,
-            };
-            $content->{exception} = $self->exception
-              if $self->has_exception;
-            return $self->serializer->serialize($content);
-        }
-
-        # Otherwise we check for a template, for a static file and,
-        # if all else fail, the default error page
-
-        if ( $self->has_app and $self->template ) {
-            # Render the template using apps' template engine.
-            # This may well be what caused the initial error, in which
-            # case we fall back to static page if any error was thrown.
-            # Note: this calls before/after render hooks.
-            my $content = eval {
-                $self->app->template(
-                    $self->template,
-                    {   title     => $self->title,
-                        content   => $self->message,
-                        exception => $self->exception,
-                        status    => $self->status,
-                    }
-                );
-            };
-            # return rendered content unless there was an error.
-            defined $content && return $content;
-        }
-
-        # It doesn't make sense to return a static page if show_errors is on
-        if ( !$self->show_errors && (my $content = $self->static_page) ) {
-            return $content;
-        }
-
-        return $self->default_error_page;
-    },
+    builder => '_build_content',
 );
 
+sub _build_content {
+    my $self = shift;
+
+    # return a hashref if a serializer is available
+    if ( $self->serializer ) {
+        my $content = {
+            message => $self->message,
+            title   => $self->title,
+            status  => $self->status,
+        };
+        $content->{exception} = $self->exception
+          if $self->has_exception;
+        return $content;
+    }
+
+    # otherwise we check for a template, for a static file,
+    # for configured error_template, and, if all else fails,
+    # the default error page
+    if ( $self->has_app and $self->template ) {
+        # Render the template using apps' template engine.
+        # This may well be what caused the initial error, in which
+        # case we fall back to static page if any error was thrown.
+        # Note: this calls before/after render hooks.
+        my $content = eval {
+            $self->app->template(
+                $self->template,
+                {   title     => $self->title,
+                    content   => $self->message,
+                    exception => $self->exception,
+                    status    => $self->status,
+                }
+            );
+        };
+
+        # return rendered content unless there was an error.
+        return $content if defined $content;
+    }
+
+    # It doesn't make sense to return a static page if show_errors is on
+    if ( !$self->show_errors && (my $content = $self->static_page) ) {
+        return $content;
+    }
+
+    if ($self->has_app && $self->app->config->{error_template}) {
+        my $content = eval {
+            $self->app->template(
+                $self->app->config->{error_template},
+                {   title     => $self->title,
+                    content   => $self->message,
+                    exception => $self->exception,
+                    status    => $self->status,
+                }
+            );
+        };
+
+        # return rendered content unless there was an error.
+        return $content if defined $content;
+    }
+
+    return $self->default_error_page;
+}
+
 sub throw {
     my $self = shift;
     $self->set_response(shift) if @_;
@@ -284,6 +308,7 @@ sub throw {
     $self->response->status( $self->status );
     $self->response->content_type( $self->content_type );
     $self->response->content($message);
+    $self->response->encode_content;
 
     $self->has_app &&
         $self->app->execute_hook('core.error.after', $self->response);
@@ -464,24 +489,6 @@ sub _html_encode {
     return $value;
 }
 
-sub _render_html {
-    my $self = shift;
-
-    # error_template defaults to something, always
-    my $template_name = $self->error_template;
-
-    my $ops = {
-        title   => $self->title,
-        content => $self->message,
-        status  => $self->status,
-        defined $self->exception ? ( exception => $self->exception ) : (),
-    };
-    my $content = $self->template->apply_renderer( $template_name, $ops );
-    $self->response->status( $self->status );
-    $self->response->header( 'Content-Type' => 'text/html' );
-    return $content;
-}
-
 1;
 
 __END__
@@ -496,7 +503,7 @@ Dancer2::Core::Error - Class representing fatal errors
 
 =head1 VERSION
 
-version 0.157001
+version 0.159000
 
 =head1 SYNOPSIS
 
@@ -606,7 +613,7 @@ Dancer Core Developers
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Alexis Sukrieh.
+This software is copyright (c) 2015 by Alexis Sukrieh.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
