@@ -1,11 +1,11 @@
 # ABSTRACT: Class to ease manipulation of MIME types
 
 package Dancer2::Core::MIME;
-$Dancer2::Core::MIME::VERSION = '0.158000';
+$Dancer2::Core::MIME::VERSION = '0.159000';
 use Moo;
 
-use MIME::Types;
-
+use Class::Load 'try_load_class';
+use Plack::MIME;
 use Dancer2::Core::Types;
 
 # Initialise MIME::Types at compile time, to ensure it's done before
@@ -14,16 +14,16 @@ use Dancer2::Core::Types;
 # as MIME::Types fails to load its mappings from the DATA handle. See
 # t/04_static_file/003_mime_types_reinit.t and GH#136.
 BEGIN {
-    MIME::Types->new( only_complete => 1 );
+    if ( try_load_class('MIME::Types') ) {
+        my $mime_types = MIME::Types->new(only_complete => 1);
+        Plack::MIME->set_fallback(
+            sub {
+                $mime_types->mimeTypeOf($_[0])
+            }
+        );
+    }
 }
 
-has mime_type => (
-    is      => 'ro',
-    isa     => InstanceOf ['MIME::Types'],
-    default => sub { MIME::Types->new( only_complete => 1 ) },
-    lazy    => 1,
-);
-
 has custom_types => (
     is      => 'ro',
     isa     => HashRef,
@@ -70,9 +70,10 @@ sub name_or_type {
 
 sub for_name {
     my ( $self, $name ) = @_;
+
     return
          $self->custom_types->{ lc $name }
-      || $self->mime_type->mimeTypeOf( lc $name )
+      || Plack::MIME->mime_type( lc ".$name" )
       || $self->default;
 }
 
@@ -90,7 +91,7 @@ Dancer2::Core::MIME - Class to ease manipulation of MIME types
 
 =head1 VERSION
 
-version 0.158000
+version 0.159000
 
 =head1 SYNOPSIS
 
@@ -127,10 +128,6 @@ for MIME handling.
 
 =head1 ATTRIBUTES
 
-=head2 mime_type
-
-The mime_type which is found with MIME::Types.
-
 =head2 custom_types
 
 Custom user-defined MIME types that are added the with C<add_type>.
