@@ -1,7 +1,7 @@
 # ABSTRACT: Response object for Dancer2
 
 package Dancer2::Core::Response;
-$Dancer2::Core::Response::VERSION = '0.158000';
+$Dancer2::Core::Response::VERSION = '0.159000';
 use Moo;
 
 use Encode;
@@ -14,7 +14,10 @@ use overload
   '@{}' => sub { $_[0]->to_psgi },
   '""'  => sub { $_[0] };
 
-with 'Dancer2::Core::Role::Headers';
+with qw<
+    Dancer2::Core::Role::Response
+    Dancer2::Core::Role::Headers
+>;
 
 # boolean to tell if the route passes or not
 has has_passed => (
@@ -206,7 +209,7 @@ Dancer2::Core::Response - Response object for Dancer2
 
 =head1 VERSION
 
-version 0.158000
+version 0.159000
 
 =head1 ATTRIBUTES
 
