@@ -1,6 +1,6 @@
 package Dancer2::Core::Role::Logger;
 # ABSTRACT: Role for logger engines
-$Dancer2::Core::Role::Logger::VERSION = '0.158000';
+$Dancer2::Core::Role::Logger::VERSION = '0.159000';
 use Dancer2::Core::Types;
 
 use Moo::Role;
@@ -118,7 +118,7 @@ sub format_message {
         f => sub { $stack[1] || '-' },
         l => sub { $stack[2] || '-' },
         h => sub {
-            ( $request && $request->remote_host || $request->address ) || '-'
+            ( $request && ( $request->remote_host || $request->address ) ) || '-'
         },
         i => sub { ( $request && $request->id ) || '-' },
     };
@@ -196,7 +196,7 @@ Dancer2::Core::Role::Logger - Role for logger engines
 
 =head1 VERSION
 
-version 0.158000
+version 0.159000
 
 =head1 DESCRIPTION
 
