@@ -1,9 +1,12 @@
 package Dancer2::Core::Route;
 # ABSTRACT: Dancer2's route handler
-$Dancer2::Core::Route::VERSION = '0.157001';
+$Dancer2::Core::Route::VERSION = '0.159000';
 use Moo;
 use Dancer2::Core::Types;
 use Carp 'croak';
+use Scalar::Util 'blessed';
+
+our ( $REQUEST, $RESPONSE, $RESPONDER, $WRITER );
 
 has method => (
     is       => 'ro',
@@ -126,8 +129,42 @@ sub match {
 }
 
 sub execute {
-    my ( $self, @args ) = @_;
-    return $self->code->(@args);
+    my ( $self, $app, @args ) = @_;
+    local $REQUEST  = $app->request;
+    local $RESPONSE = $app->response;
+
+    my $content = $self->code->( $app, @args );
+
+    # a user might have set the content in the response,
+    # so we ignore the return value and use that content instead
+    # but we only override if there was no object returned
+    $RESPONSE->has_content && !ref $content
+        and $content = $RESPONSE->content;
+
+    my $type = blessed($content)
+        or return $app->_add_content_to_response( $RESPONSE, $content );
+
+    # Plack::Response: proper ArrayRef-style response
+    $type eq 'Plack::Response'
+        and $RESPONSE = Dancer2::Core::Response->new_from_plack($RESPONSE);
+
+    # CodeRef: raw PSGI response
+    # do we want to allow it and forward it back?
+    # do we want to upgrade it to an asynchronous response?
+    $type eq 'CODE'
+        and die "We do not support returning code references from routes.\n";
+
+    # Dancer2::Core::Response, Dancer2::Core::Response::Delayed:
+    # proper responses
+    $type eq 'Dancer2::Core::Response'
+        and return $RESPONSE;
+
+    $type eq 'Dancer2::Core::Response::Delayed'
+        and return $content;
+
+    # we can't handle arrayref or hashref
+    # because those might be serialized back
+    die "Unrecognized response type from route: $type.\n";
 }
 
 # private subs
@@ -223,7 +260,7 @@ Dancer2::Core::Route - Dancer2's route handler
 
 =head1 VERSION
 
-version 0.157001
+version 0.159000
 
 =head1 ATTRIBUTES
 
@@ -268,7 +305,7 @@ Dancer Core Developers
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Alexis Sukrieh.
+This software is copyright (c) 2015 by Alexis Sukrieh.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
