@@ -1,9 +1,10 @@
 package Dancer2::Plugin;
 # ABSTRACT: Extending Dancer2's DSL with plugins
-$Dancer2::Plugin::VERSION = '0.157001';
+$Dancer2::Plugin::VERSION = '0.159000';
 use Moo::Role;
 use Carp 'croak', 'carp';
 use Dancer2::Core::DSL;
+use Scalar::Util qw();
 
 # singleton for storing all keywords,
 # their code and the plugin they come from
@@ -39,7 +40,7 @@ sub register {
 
     $_keywords->{$plugin} ||= [];
     push @{ $_keywords->{$plugin} },
-      [ $keyword, $code, $options->{is_global} ];
+      [ $keyword, $code, $options ];
 }
 
 sub on_plugin_import(&) {
@@ -63,10 +64,10 @@ sub register_plugin {
     # bind all registered keywords to the plugin
     my $dsl = $caller->dsl;
     for my $k ( @{ $_keywords->{$plugin} } ) {
-        my ( $keyword, $code, $is_global ) = @{$k};
+        my ( $keyword, $code, $options ) = @{$k};
         {
             no strict 'refs';
-            *{"${plugin}::${keyword}"} = $code;
+            *{"${plugin}::${keyword}"} = $dsl->_apply_prototype($code, $options);
         }
     }
 
@@ -79,7 +80,8 @@ sub register_plugin {
         my $caller = caller(1);
 
         for my $k ( @{ $_keywords->{$plugin} } ) {
-            my ( $keyword, $code, $is_global ) = @{$k};
+            my ( $keyword, $code, $options ) = @{$k};
+            my $is_global = exists $options->{is_global} && $options->{is_global};
             $caller->dsl->register( $keyword, $is_global );
         }
 
@@ -213,8 +215,12 @@ sub import {
                 $code->( $dsl, @_ );
             };
 
-            # bind the newly compiled symbol to the caller's namespace.
-            *{"${plugin}::${symbol}"} = $compiled;
+            # Bind the newly compiled symbol to the caller's namespace.
+            # As this may redefine a symbol, ensure the new coderef has
+            # the same prototype signature.
+            my $existing = *{"${plugin}::${symbol}"};
+            my $prototype = prototype \&$existing;
+            *{"${plugin}::${symbol}"} = Scalar::Util::set_prototype( \&$compiled, $prototype );
 
             $dsl_deprecation_wrapper = $compiled if $symbol eq 'dsl';
         }
@@ -253,7 +259,7 @@ Dancer2::Plugin - Extending Dancer2's DSL with plugins
 
 =head1 VERSION
 
-version 0.157001
+version 0.159000
 
 =head1 DESCRIPTION
 
@@ -441,7 +447,7 @@ Dancer Core Developers
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Alexis Sukrieh.
+This software is copyright (c) 2015 by Alexis Sukrieh.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
